import re

expressions = \
    {
        'True':
            {
                r"\b(owo|uwu)\b",
                r"\b[oо0u🇴🇺]+[w🇼]+[oо0u🇴🇺]+\b",
                r"\b[oо0>u🇴🇺^]+[\s.,*_-`\\]*[w3🇼]+[\s,.*_-`\\]*[oо0^<u🇴🇺]\b"
            },
        'False':
            {
                r"(owo|uwu)",
                r"[oо0u🇴🇺]+[w🇼]+[oо0u🇴🇺]",
                r"[oо0>u🇴🇺^]+[\s.,*_-`\\]*[w3🇼]+[\s,.*_-`\\]*[oо0^<u🇴🇺]"
            }
    }

def __message_is_valid(message: str) -> bool:
    """Returns True if the message is valid.

    :param message: message to be evaluated
    :return: True if the message is valid
    """

    # Ignore empty messages
    if len(message) < 3 or message.isspace():
        return False

    return True

def fast_evaluate(message: str) -> bool:
    """Returns True if the message contains an OwO-like expression.

    Fast evaluate checks only if the message contains the exact
    characters 'owo' and 'uwu', and therefore is much less accurate
    than the normal evaluate function.

    :param message: message to be evaluated
    :return: True if the message contains an 'owo'-like expression
    """

    # Ignore empty messages
    if not __message_is_valid(message):
        return False

    if 'owo' in message.lower() or 'uwu' in message.lower():
        return True

    return False


def evaluate(message: str, word_boundaries: bool = False) -> bool:
    """Returns True if the message contains an OwO-like expression.

    :param word_boundaries: enable/disable boundary checks
    :param message: message to be evaluated
    :return: True if the message contains an 'owo'-like expression
    """

    # Ignore empty messages
    if not __message_is_valid:
        return False

    for expression in expressions[str(word_boundaries)]:
        if re.search(expression, message, re.IGNORECASE):
            return True

    return False



import is_furry

tests: list[tuple[str, bool, bool]] = \
    [
        ("hello", False, False),
        ("hello owo", True, False),
        ("hello uwu", True, False),
        ("hello owo uwu", True, False),
        ("hello nowo", True, False),
        ("hello nowo", False, True),
        ("hello owo", True, False),
        ("Hello OwO", True, False),
        ("^w^", False, True),
        ("OwO~ x3", True, False),
    ]

fast_tests: list[tuple[str, bool]] = \
    [
        ("hello", False),
        ("hello owo", True),
        ("hello uwu", True),
        ("hello owouwu", True),
        ("hello o w o", False),
    ]


for test in tests:
    if len(test) > 2:
        result = is_furry.evaluate(test[0], test[2])
    else:
        result = is_furry.evaluate(test[0])

    if result != test[1]:
        print(f"Test failed: {test[0]}")
        print(f"Expected: {test[1]}")
        print(f"Actual: {result}")
        exit(1)
    else:
        print(f"Test passed: {test[0]}")

for test in fast_tests:
    result = is_furry.fast_evaluate(test[0])

    if result != test[1]:
        print(f"Test failed: {test[0]}")
        print(f"Expected: {test[1]}")
        print(f"Actual: {result}")
        exit(1)
    else:
        print(f"Test passed: {test[0]}")